#include <cmath>

#include "qthing/qthing_legacy.hpp"

namespace qthing {
// TODO: should return RGB once that can be casted to RGBW
RGB hue(const float hue, const float value) {
  if (value <= 0) {
    return RGB();
  }

  auto h = fmod(hue, 1);
  if (h < 0) {
    h += 1;
  }

  h = h * 3;
  auto v = value > 1 ? 1 : value;
  if (h < 1) {
    return RGB((1 - h) * v, h * v, 0);
  } else if (h < 2) {
    h -= 1;
    return RGB(0, (1 - h) * v, h * v);
  } else {
    h -= 2;
    return RGB(h * v, 0, (1 - h) * v);
  }
}

RGB hsv(float hue, float saturation, float value) {
  if (value <= 0) return RGB();

  if (saturation <= 0) return RGB(value, value, value);

  if (saturation > 1) saturation = 1;

  if (value > 1) value = 1;

  hue = fmod(hue, 1);
  if (hue < 0) ++hue;

  float hh = hue * 6;
  int i = (int)hh;
  float ff = hh - i;
  float p = value * (1.0 - saturation);
  float q = value * (1.0 - (saturation * ff));
  float t = value * (1.0 - (saturation * (1.0 - ff)));

  switch (i) {
    case (0):
      return RGB(value, t, p);
    case (1):
      return RGB(q, value, p);
    case (2):
      return RGB(p, value, t);
    case (3):
      return RGB(p, q, value);
    case (4):
      return RGB(t, p, value);
    case (5):
      return RGB(value, p, q);
  }
  return RGB();
}
}  // namespace qthing
