#pragma once

#include <cstddef>
#include <functional>
#include <vector>

namespace qthing {

template <class, class = void>
class Callback; /* undefined */

template <typename... Args>
class Callback<void(Args...), void> {
 private:
  using CallbackType = std::function<void(Args...)>;
  mutable CallbackType callback;

 public:
  Callback() : callback(NULL){};

  void add(CallbackType newCallback) const {
    if (callback == NULL) {
      callback = newCallback;
    } else {
      auto oldCallback = callback;
      callback = [&](Args&&... args) {
        oldCallback(std::forward<Args>(args)...);
        newCallback(std::forward<Args>(args)...);
      };
    }
  }
  auto& operator+=(const CallbackType& rhs) const {
    add(rhs);
    return *this;
  }
  void operator()(Args&&... args) const {
    if (callback != NULL) {
      callback(std::forward<Args>(args)...);
    }
  }
};

template <typename R, typename... Args>
class Callback<R(Args...), void> {
 private:
  using CallbackType = std::function<R(Args...)>;
  mutable std::function<void(std::vector<R>&, Args...)> callback;

 public:
  Callback() : callback(NULL){};

  void add(CallbackType newCallback) const {
    if (callback == NULL) {
      callback = [&](std::vector<R> results, auto&&... args) {
        results.push_back(newCallback(std::forward<Args>(args)...));
      };
    } else {
      std::function<void(std::vector<R>&, Args...)> oldCallback = callback;
      callback = [&](std::vector<R> results, auto&&... args) {
        oldCallback(std::forward<Args>(args)...);
        results.push_back(newCallback(std::forward<Args>(args)...));
      };
    }
  }
  auto& operator+=(const CallbackType& rhs) const {
    add(rhs);
    return *this;
  }
  std::vector<R> operator()(Args&&... args) const {
    std::vector<R> results;
    if (callback != NULL) {
      callback(results, std::forward<Args>(args)...);
    }
    return results;
  }
};

template <typename FriendClass, typename R, typename... Args>
class Callback<R(Args...), FriendClass> : public Callback<R(Args...)> {
  friend FriendClass;

 private:
  auto operator()(Args&&... args) const { Callback<R(Args...)>::operator()(std::forward<Args>(args)...); };
};

}  // namespace qthing
