#pragma once

#include <esp_log.h>

#include "device_config.h"
#include "qthing/core.hpp"
#include "qthing/description.hpp"

namespace qthing {

class DescriptionDriver : public Driver {
  friend DriverStorage<DescriptionDriver>;

  DescriptionDriver(const Driver&) = delete;             // delete cctor
  DescriptionDriver& operator=(const Driver&) = delete;  // delete cassop
  DescriptionDriver(Driver&&) = delete;                  // delete mctor
  DescriptionDriver& operator=(Driver&&) = delete;       // delete massop

 private:
  std::string description;
  DescriptionDriver() {  // declare ctor private
    getDriver<Core>()->onActiveConfigChanged += [&](auto& config) { updateDescription(config); };
    updateDescription();
  }
  std::string generateDescription(const Config& config) {
    DeviceDescription deviceDescription(DEVICE_NAME);
    for (auto&& component : config.components()) {
      component->describe(deviceDescription);
    }
    return deviceDescription.encode();
  }
  void updateDescription(const Config& config = getDriver<Core>()->getActiveConfig()) {
    std::string oldDescription(description);
    description = generateDescription(config);
    ESP_LOGI("DescriptionDriver", "Description generated: %s", description.c_str());
    if (description != oldDescription) {
      onDescriptionChanged(description);
    }
  }

 public:
  const Callback<void(const std::string&), DescriptionDriver> onDescriptionChanged;
  const std::string& get() { return description; }
};

}  // namespace qthing
