#pragma once

#include <cmath>

#include "qthing/core.hpp"
#include "qthing/mqtt_ota.hpp"
#include "qthing/util.hpp"

namespace qthing {

enum class OtaLedBehavior {
  otaStatus,
  off,
  normal,
};

class OtaDriver : public Driver {
  friend DriverStorage<OtaDriver>;
  OtaDriver(const OtaDriver&) = delete;             // delete cctor
  OtaDriver& operator=(const OtaDriver&) = delete;  // delete cassop
  OtaDriver(OtaDriver&&) = delete;                  // delete mctor
  OtaDriver& operator=(OtaDriver&&) = delete;       // delete massop

 private:
  float otaProgress = 0;
  OtaDriver() {
    add_ota_event_callback([&](ota_event_t event) { otaProgress = (float)event.bytes_written / event.bytes_total; });
  }

 public:
  std::function<qthing::RGBW8(uint16_t address)> otaColorHandler() {
    return [&](uint16_t address) {
      return static_cast<RGBW8>(hue(otaProgress, sin(millis() / 1000. * 5) * 0.3 + 0.7));
    };
  }
};

}  // namespace qthing
